@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -52,16 +52,20 @@ int lxc_af_unix_open(const char *path, int type, int flags)
 
 	addr.sun_family = AF_UNIX;
 	/* copy entire buffer in case of abstract socket */
-	memcpy(addr.sun_path, path, 
+	memcpy(addr.sun_path, path,
 	       path[0]?strlen(path):sizeof(addr.sun_path));
 
 	if (bind(fd, (struct sockaddr *)&addr, sizeof(addr))) {
+		int tmp = errno;
 		close(fd);
+		errno = tmp;
 		return -1;
 	}
 	
 	if (type == SOCK_STREAM && listen(fd, 100)) {
+		int tmp = errno;
 		close(fd);
+		errno = tmp;
 		return -1;
 	}
 
@@ -73,7 +77,7 @@ int lxc_af_unix_close(int fd)
 	struct sockaddr_un addr;
 	socklen_t addrlen;
 	
-	if (!getsockname(fd, (struct sockaddr *)&addr, &addrlen) && 
+	if (!getsockname(fd, (struct sockaddr *)&addr, &addrlen) &&
 	    addr.sun_path[0])
 		unlink(addr.sun_path);
 
@@ -95,11 +99,13 @@ int lxc_af_unix_connect(const char *path)
 
 	addr.sun_family = AF_UNIX;
 	/* copy entire buffer in case of abstract socket */
-	memcpy(addr.sun_path, path, 
+	memcpy(addr.sun_path, path,
 	       path[0]?strlen(path):sizeof(addr.sun_path));
 
 	if (connect(fd, (struct sockaddr *)&addr, sizeof(addr))) {
+		int tmp = errno;
 		close(fd);
+		errno = tmp;
 		return -1;
 	}
 
@@ -113,6 +119,7 @@ int lxc_af_unix_send_fd(int fd, int sendfd, void *data, size_t size)
         struct cmsghdr *cmsg;
         char cmsgbuf[CMSG_SPACE(sizeof(int))];
         char buf[1];
+	int *val;
 
         msg.msg_control = cmsgbuf;
         msg.msg_controllen = sizeof(cmsgbuf);
@@ -121,7 +128,8 @@ int lxc_af_unix_send_fd(int fd, int sendfd, void *data, size_t size)
         cmsg->cmsg_len = CMSG_LEN(sizeof(int));
         cmsg->cmsg_level = SOL_SOCKET;
         cmsg->cmsg_type = SCM_RIGHTS;
-        *((int *) CMSG_DATA(cmsg)) = sendfd;
+	val = (int *)(CMSG_DATA(cmsg));
+	*val = sendfd;
 
         msg.msg_name = NULL;
         msg.msg_namelen = 0;
@@ -141,7 +149,7 @@ int lxc_af_unix_recv_fd(int fd, int *recvfd, void *data, size_t size)
         struct cmsghdr *cmsg;
         char cmsgbuf[CMSG_SPACE(sizeof(int))];
         char buf[1];
-	int ret;
+	int ret, *val;
 
         msg.msg_name = NULL;
         msg.msg_namelen = 0;
@@ -159,14 +167,15 @@ int lxc_af_unix_recv_fd(int fd, int *recvfd, void *data, size_t size)
 
         cmsg = CMSG_FIRSTHDR(&msg);
 
-	/* if the message is wrong the variable will not be 
+	/* if the message is wrong the variable will not be
 	 * filled and the peer will notified about a problem */
 	*recvfd = -1;
 
         if (cmsg && cmsg->cmsg_len == CMSG_LEN(sizeof(int)) &&
             cmsg->cmsg_level == SOL_SOCKET &&
             cmsg->cmsg_type == SCM_RIGHTS) {
-                *recvfd = *((int *) CMSG_DATA(cmsg));
+		val = (int *) CMSG_DATA(cmsg);
+                *recvfd = *val;
         }
 out:
         return ret;
@@ -192,7 +201,7 @@ int lxc_af_unix_send_credential(int fd, void *data, size_t size)
         cmsg->cmsg_len = CMSG_LEN(sizeof(struct ucred));
         cmsg->cmsg_level = SOL_SOCKET;
         cmsg->cmsg_type = SCM_CREDENTIALS;
-	*((struct ucred *) CMSG_DATA(cmsg)) = cred;
+	memcpy(CMSG_DATA(cmsg), &cred, sizeof(cred));
 
         msg.msg_name = NULL;
         msg.msg_namelen = 0;
@@ -234,7 +243,7 @@ int lxc_af_unix_rcv_credential(int fd, void *data, size_t size)
         if (cmsg && cmsg->cmsg_len == CMSG_LEN(sizeof(struct ucred)) &&
             cmsg->cmsg_level == SOL_SOCKET &&
             cmsg->cmsg_type == SCM_CREDENTIALS) {
-                cred = *((struct ucred *) CMSG_DATA(cmsg));
+		memcpy(&cred, CMSG_DATA(cmsg), sizeof(cred));
 		if (cred.uid && (cred.uid != getuid() || cred.gid != getgid())) {
 			INFO("message denied for '%d/%d'", cred.uid, cred.gid);
 			return -EACCES;
