@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  * Michel Normand <normand at fr.ibm.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -32,6 +32,7 @@
 #include <unistd.h>
 
 #include "arguments.h"
+#include "utils.h"
 
 /*---------------------------------------------------------------------------*/
 static int build_shortopts(const struct option *a_options,
@@ -96,6 +97,9 @@ static void print_usage(const struct option longopts[],
 		int j;
 		char *uppername = strdup(opt->name);
 
+		if (!uppername)
+			exit(-ENOMEM);
+
 		for (j = 0; uppername[j]; j++)
 			uppername[j] = toupper(uppername[j]);
 
@@ -133,6 +137,7 @@ Common options :\n\
   -o, --logfile=FILE               Output log to FILE instead of stderr\n\
   -l, --logpriority=LEVEL          Set log priority to LEVEL\n\
   -q, --quiet                      Don't produce any output\n\
+  -P, --lxcpath=PATH               Use specified container path\n\
   -?, --help                       Give this help list\n\
       --usage                      Give a short usage message\n\
 \n\
@@ -151,6 +156,7 @@ extern int lxc_arguments_parse(struct lxc_arguments *args,
 	char shortopts[256];
 	int  ret = 0;
 
+	args->lxcpath = default_lxc_path();
 	ret = build_shortopts(args->options, shortopts, sizeof(shortopts));
 	if (ret < 0) {
 		lxc_error(args, "build_shortopts() failed : %s",
@@ -170,6 +176,7 @@ extern int lxc_arguments_parse(struct lxc_arguments *args,
 		case 'l':	args->log_priority = optarg; break;
 		case 'c':	args->console = optarg; break;
 		case 'q':	args->quiet = 1; break;
+		case 'P':	args->lxcpath = optarg; break;
 		case OPT_USAGE: print_usage(args->options, args);
 		case '?':	print_help(args, 1);
 		case 'h': 	print_help(args, 0);
@@ -203,35 +210,6 @@ error:
 	return ret;
 }
 
-extern char **lxc_arguments_dup(const char *file, struct lxc_arguments *args)
-{
-	char **argv;
-	int opt, nbargs = args->argc + 2;
-
-	if (args->quiet)
-		nbargs += 1;
-
-	argv = malloc((nbargs + 1) * sizeof(*argv));
-	if (!argv)
-		return NULL;
-
-	nbargs = 0;
-
-	argv[nbargs++] = strdup(file);
-
-	if (args->quiet)
-		argv[nbargs++] = "--quiet";
-
-	argv[nbargs++] = "--";
-
-	for (opt = 0; opt < args->argc; opt++)
-		argv[nbargs++] = strdup(args->argv[opt]);
-
-	argv[nbargs] = NULL;
-
-	return argv;
-}
-
 int lxc_arguments_str_to_int(struct lxc_arguments *args, const char *str)
 {
 	long val;
