@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  * Michel Normand <normand at fr.ibm.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -45,6 +45,9 @@ struct lxc_arguments {
 	int daemonize;
 	const char *rcfile;
 	const char *console;
+	const char *console_log;
+	const char *pidfile;
+	const char *lxcpath;
 
 	/* for lxc-checkpoint/restart */
 	const char *statefile;
@@ -57,10 +60,17 @@ struct lxc_arguments {
 
 	/* for lxc-wait */
 	char *states;
+	long timeout;
+
+	/* close fds from parent? */
+	int close_all_fds;
 
 	/* remaining arguments */
 	char *const *argv;
 	int argc;
+
+	/* private arguments */
+	void *data;
 };
 
 #define LXC_COMMON_OPTIONS \
@@ -70,6 +80,7 @@ struct lxc_arguments {
 	{"quiet", no_argument,	0, 'q'}, \
 	{"logfile", required_argument, 0, 'o'}, \
 	{"logpriority", required_argument, 0, 'l'}, \
+	{"lxcpath", required_argument, 0, 'P'}, \
 	{0, 0, 0, 0}
 
 /* option keys for long only options */
@@ -78,7 +89,6 @@ struct lxc_arguments {
 extern int lxc_arguments_parse(struct lxc_arguments *args,
 			       int argc, char *const argv[]);
 
-extern char **lxc_arguments_dup(const char *file, struct lxc_arguments *args);
 extern int lxc_arguments_str_to_int(struct lxc_arguments *args, const char *str);
 
 extern const char *lxc_strerror(int errnum);
