@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,37 +20,66 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
+#include "config.h"
+
 #ifndef _caps_h
 #define _caps_h
 
+#if HAVE_SYS_CAPABILITY_H
 extern int lxc_caps_reset(void);
 extern int lxc_caps_down(void);
 extern int lxc_caps_up(void);
 extern int lxc_caps_init(void);
+extern int lxc_caps_check(void);
+
+extern int lxc_caps_last_cap(void);
+#else
+static inline int lxc_caps_reset(void) {
+        return 0;
+}
+static inline int lxc_caps_down(void) {
+        return 0;
+}
+static inline int lxc_caps_up(void) {
+        return 0;
+}
+static inline int lxc_caps_init(void) {
+        return 0;
+}
+static inline int lxc_caps_check(void) {
+        return 1;
+}
+
+static inline int lxc_caps_last_cap(void) {
+        return 0;
+}
+#endif
 
 #define lxc_priv(__lxc_function)			\
 	({						\
-		int __ret, __ret2, __errno = 0;		\
+		__label__ out;				\
+		int __ret, __ret2, ___errno = 0;		\
 		__ret = lxc_caps_up();			\
 		if (__ret)				\
-			goto __out;			\
+			goto out;			\
 		__ret = __lxc_function;			\
 		if (__ret)				\
-			__errno = errno;		\
+			___errno = errno;		\
 		__ret2 = lxc_caps_down();		\
-	__out:	__ret ? errno = __errno,__ret : __ret2;	\
+	out:	__ret ? errno = ___errno,__ret : __ret2;	\
 	})
 
-#define lxc_unpriv(__lxc_function)		\
+#define lxc_unpriv(__lxc_function)			\
 	({						\
-		int __ret, __ret2, __errno = 0;		\
+		__label__ out;				\
+		int __ret, __ret2, ___errno = 0;		\
 		__ret = lxc_caps_down();		\
 		if (__ret)				\
-			goto __out;			\
+			goto out;			\
 		__ret = __lxc_function;			\
 		if (__ret)				\
-			__errno = errno;		\
+			___errno = errno;		\
 		__ret2 = lxc_caps_up();			\
-	__out:	__ret ? errno = __errno,__ret : __ret2;	\
+	out:	__ret ? errno = ___errno,__ret : __ret2;	\
 	})
 #endif
