@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2009
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,8 @@ enum {
 	LXC_COMMAND_STOP,
 	LXC_COMMAND_STATE,
 	LXC_COMMAND_PID,
+	LXC_COMMAND_CLONE_FLAGS,
+	LXC_COMMAND_CGROUP,
 	LXC_COMMAND_MAX,
 };
 
@@ -40,6 +42,8 @@ struct lxc_answer {
 	int fd;
 	int ret; /* 0 on success, -errno on failure */
 	pid_t pid;
+	int pathlen;
+	const char *path;
 };
 
 struct lxc_command {
@@ -47,17 +51,20 @@ struct lxc_command {
 	struct lxc_answer answer;
 };
 
-extern pid_t get_init_pid(const char *name);
+extern pid_t get_init_pid(const char *name, const char *lxcpath);
+extern int lxc_get_clone_flags(const char *name, const char *lxcpath);
 
 extern int lxc_command(const char *name, struct lxc_command *command,
-			int *stopped);
+			int *stopped, const char *lxcpath);
 
 extern int lxc_command_connected(const char *name, struct lxc_command *command,
-				 int *stopped);
+				 int *stopped, const char *lxcpath);
 
 struct lxc_epoll_descr;
 struct lxc_handler;
 
+extern int lxc_command_init(const char *name, struct lxc_handler *handler,
+			    const char *lxcpath);
 extern int lxc_command_mainloop_add(const char *name, struct lxc_epoll_descr *descr,
 				    struct lxc_handler *handler);
 
