@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,14 +23,22 @@
 #ifndef _conf_h
 #define _conf_h
 
+#include "config.h"
+
 #include <netinet/in.h>
+#include <net/if.h>
 #include <sys/param.h>
+#include <sys/types.h>
 #include <stdbool.h>
 
 #include <lxc/list.h>
 
 #include <lxc/start.h> /* for lxc_handler */
 
+#if HAVE_SCMP_FILTER_CTX
+typedef void * scmp_filter_ctx;
+#endif
+
 enum {
 	LXC_NET_EMPTY,
 	LXC_NET_VETH,
@@ -76,13 +84,14 @@ struct lxc_route6 {
 
 struct ifla_veth {
 	char *pair; /* pair name */
+	char veth1[IFNAMSIZ]; /* needed for deconf */
 };
 
 struct ifla_vlan {
 	uint   flags;
 	uint   fmask;
-	ushort   vid;
-	ushort   pad;
+	unsigned short   vid;
+	unsigned short   pad;
 };
 
 struct ifla_macvlan {
@@ -103,6 +112,7 @@ union netdev_p {
  * @ipv4       : a list of ipv4 addresses to be set on the network device
  * @ipv6       : a list of ipv6 addresses to be set on the network device
  * @upscript   : a script filename to be executed during interface configuration
+ * @downscript : a script filename to be executed during interface destruction
  */
 struct lxc_netdev {
 	int type;
@@ -120,6 +130,7 @@ struct lxc_netdev {
 	struct in6_addr *ipv6_gateway;
 	bool ipv6_gateway_auto;
 	char *upscript;
+	char *downscript;
 };
 
 /*
@@ -133,6 +144,26 @@ struct lxc_cgroup {
 	char *value;
 };
 
+enum idtype {
+	ID_TYPE_UID,
+	ID_TYPE_GID
+};
+
+/*
+ * id_map is an id map entry.  Form in confile is:
+ * lxc.id_map = u 0    9800 100
+ * lxc.id_map = u 1000 9900 100
+ * lxc.id_map = g 0    9800 100
+ * lxc.id_map = g 1000 9900 100
+ * meaning the container can use uids and gids 0-99 and 1000-1099,
+ * with [ug]id 0 mapping to [ug]id 9800 on the host, and [ug]id 1000 to
+ * [ug]id 9900 on the host.
+ */
+struct id_map {
+	enum idtype idtype;
+	unsigned long hostid, nsid, range;
+};
+
 /*
  * Defines a structure containing a pty information for
  * virtualizing a tty
@@ -167,6 +198,8 @@ struct lxc_console {
 	int master;
 	int peer;
 	char *path;
+	char *log_path;
+	int log_fd;
 	char name[MAXPATHLEN];
 	struct termios *tios;
 };
@@ -203,8 +236,15 @@ struct lxc_rootfs {
 #endif
  */
 enum lxchooks {
-	LXCHOOK_PRESTART, LXCHOOK_MOUNT, LXCHOOK_START,
-	LXCHOOK_POSTSTOP, NUM_LXC_HOOKS};
+	LXCHOOK_PRESTART, LXCHOOK_PREMOUNT, LXCHOOK_MOUNT, LXCHOOK_AUTODEV,
+	LXCHOOK_START, LXCHOOK_POSTSTOP, NUM_LXC_HOOKS};
+extern char *lxchook_names[NUM_LXC_HOOKS];
+
+struct saved_nic {
+	int ifindex;
+	char *orig_name;
+};
+
 struct lxc_conf {
 	char *fstab;
 	int tty;
@@ -214,7 +254,10 @@ struct lxc_conf {
 	int personality;
 	struct utsname *utsname;
 	struct lxc_list cgroup;
+	struct lxc_list id_map;
 	struct lxc_list network;
+	struct saved_nic *saved_nics;
+	int num_savednics;
 	struct lxc_list mount_list;
 	struct lxc_list caps;
 	struct lxc_tty_info tty_info;
@@ -226,31 +269,59 @@ struct lxc_conf {
 #if HAVE_APPARMOR
 	char *aa_profile;
 #endif
+
 #if HAVE_APPARMOR /* || HAVE_SELINUX || HAVE_SMACK */
 	int lsm_umount_proc;
 #endif
+	char *seccomp;  // filename with the seccomp rules
+#if HAVE_SCMP_FILTER_CTX
+	scmp_filter_ctx *seccomp_ctx;
+#endif
+	int maincmd_fd;
+	int autodev;  // if 1, mount and fill a /dev at start
+	int stopsignal; // signal used to stop container
+	int kmsg;  // if 1, create /dev/kmsg symlink
+	char *rcfile;	// Copy of the top level rcfile we read
 };
 
 int run_lxc_hooks(const char *name, char *hook, struct lxc_conf *conf);
 
+extern int setup_cgroup(const char *cgpath, struct lxc_list *cgroups);
+extern int detect_shared_rootfs(void);
+
 /*
  * Initialize the lxc configuration structure
  */
 extern struct lxc_conf *lxc_conf_init(void);
+extern void lxc_conf_free(struct lxc_conf *conf);
 
 extern int pin_rootfs(const char *rootfs);
 
 extern int lxc_create_network(struct lxc_handler *handler);
-extern void lxc_delete_network(struct lxc_list *networks);
+extern void lxc_delete_network(struct lxc_handler *handler);
 extern int lxc_assign_network(struct lxc_list *networks, pid_t pid);
+extern int lxc_map_ids(struct lxc_list *idmap, pid_t pid);
 extern int lxc_find_gateway_addresses(struct lxc_handler *handler);
 
 extern int lxc_create_tty(const char *name, struct lxc_conf *conf);
 extern void lxc_delete_tty(struct lxc_tty_info *tty_info);
 
+extern int lxc_clear_config_network(struct lxc_conf *c);
+extern int lxc_clear_nic(struct lxc_conf *c, const char *key);
+extern int lxc_clear_config_caps(struct lxc_conf *c);
+extern int lxc_clear_cgroups(struct lxc_conf *c, const char *key);
+extern int lxc_clear_mount_entries(struct lxc_conf *c);
+extern int lxc_clear_hooks(struct lxc_conf *c, const char *key);
+
+extern int setup_cgroup(const char *name, struct lxc_list *cgroups);
+
+extern int uid_shift_ttys(int pid, struct lxc_conf *conf);
+
 /*
  * Configure the container from inside
  */
 
 extern int lxc_setup(const char *name, struct lxc_conf *lxc_conf);
+
+extern void lxc_rename_phys_nics_on_shutdown(struct lxc_conf *conf);
 #endif
