@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,6 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-
 #include "log.h"
 #include "start.h"
 
@@ -128,7 +127,7 @@ static struct lxc_operations execute_start_ops = {
 };
 
 int lxc_execute(const char *name, char *const argv[], int quiet,
-		struct lxc_conf *conf)
+		struct lxc_conf *conf, const char *lxcpath)
 {
 	struct execute_args args = {
 		.argv = argv,
@@ -138,5 +137,5 @@ int lxc_execute(const char *name, char *const argv[], int quiet,
 	if (lxc_check_inherited(conf, -1))
 		return -1;
 
-	return __lxc_start(name, conf, &execute_start_ops, &args);
+	return __lxc_start(name, conf, &execute_start_ops, &args, lxcpath);
 }
