@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  * Cedric Le Goater <legoater@free.fr>
  *
  * This library is free software; you can redistribute it and/or
@@ -41,7 +41,7 @@
 #define LXC_LOG_BUFFER_SIZE	512
 
 /* predefined priorities. */
-enum {
+enum lxc_loglevel {
 	LXC_LOG_PRIORITY_TRACE,
 	LXC_LOG_PRIORITY_DEBUG,
 	LXC_LOG_PRIORITY_INFO,
@@ -285,6 +285,14 @@ extern struct lxc_log_category lxc_log_category_lxc;
 	ERROR("%s - " format, strerror(errno), ##__VA_ARGS__);		\
 } while (0)
 
-extern int lxc_log_init(const char *file, const char *priority,
-			const char *prefix, int quiet);
+extern int lxc_log_fd;
+
+extern int lxc_log_init(const char *name, const char *file,
+			const char *priority, const char *prefix, int quiet);
+
+extern void lxc_log_setprefix(const char *a_prefix);
+extern int lxc_log_set_level(int level);
+extern int lxc_log_set_file(const char *fname);
+extern int lxc_log_get_level(void);
+extern const char *lxc_log_get_file(void);
 #endif
