@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,7 @@ extern "C" {
 
 struct lxc_msg;
 struct lxc_conf;
+struct lxc_arguments;
 
 /**
  Following code is for liblxc.
@@ -40,13 +41,14 @@ struct lxc_conf;
  **/
 
 /*
- * Start the specified command inside a container
+ * Start the specified command inside a system container
  * @name     : the name of the container
  * @argv     : an array of char * corresponding to the commande line
  * @conf     : configuration
  * Returns 0 on sucess, < 0 otherwise
  */
-extern int lxc_start(const char *name, char *const argv[], struct lxc_conf *conf);
+extern int lxc_start(const char *name, char *const argv[], struct lxc_conf *conf,
+		     const char *lxcpath);
 
 /*
  * Stop the container previously started with lxc_start, all
@@ -54,14 +56,25 @@ extern int lxc_start(const char *name, char *const argv[], struct lxc_conf *conf
  * @name : the name of the container
  * Returns 0 on success, < 0 otherwise
  */
-extern int lxc_stop(const char *name);
+extern int lxc_stop(const char *name, const char *lxcpath);
+
+/*
+ * Start the specified command inside an application container
+ * @name     : the name of the container
+ * @argv     : an array of char * corresponding to the commande line
+ * @quiet    : if != 0 then lxc-init won't produce any output
+ * @conf     : configuration
+ * Returns 0 on sucess, < 0 otherwise
+ */
+extern int lxc_execute(const char *name, char *const argv[], int quiet,
+		       struct lxc_conf *conf, const char *lxcpath);
 
 /*
  * Open the monitoring mechanism for a specific container
  * The function will return an fd corresponding to the events
  * Returns a file descriptor on success, < 0 otherwise
  */
-extern int lxc_monitor_open(void);
+extern int lxc_monitor_open(const char *lxcpath);
 
 /*
  * Read the state of the container if this one has changed
@@ -72,6 +85,7 @@ extern int lxc_monitor_open(void);
  * data was readen, < 0 otherwise
  */
 extern int lxc_monitor_read(int fd, struct lxc_msg *msg);
+extern int lxc_monitor_read_timeout(int fd, struct lxc_msg *msg, int timeout);
 
 /*
  * Close the fd associated with the monitoring
@@ -87,50 +101,70 @@ extern int lxc_monitor_close(int fd);
  * @fd   : a pointer to a tty file descriptor
  * Returns 0 on sucess, < 0 otherwise
  */
-extern int lxc_console(const char *name, int ttynum, int *fd);
+extern int lxc_console(const char *name, int ttynum, int *fd, const char *lxcpath);
 
 /*
  * Freeze all the tasks running inside the container <name>
  * @name : the container name
  * Returns 0 on success, < 0 otherwise
  */
-extern int lxc_freeze(const char *name);
+extern int lxc_freeze(const char *name, const char *lxcpath);
 
 /*
  * Unfreeze all previously frozen tasks.
  * @name : the name of the container
  * Return 0 on sucess, < 0 otherwise
  */
-extern int lxc_unfreeze(const char *name);
+extern int lxc_unfreeze(const char *name, const char *lxcpath);
+
+/*
+ * Unfreeze all previously frozen tasks.
+ * This is the function to use from inside the monitor
+ * @name : the name of the container
+ * Return 0 on sucess, < 0 otherwise
+ */
+extern int lxc_unfreeze_bypath(const char *cgpath);
 
 /*
  * Retrieve the container state
  * @name : the name of the container
  * Returns the state of the container on success, < 0 otherwise
  */
-extern lxc_state_t lxc_state(const char *name);
+extern lxc_state_t lxc_state(const char *name, const char *lxcpath);
+
+/*
+ * Set a specified value for a specified subsystem. The specified
+ * subsystem must be fully specified, eg. "cpu.shares"
+ * @cgpath    : the cgroup path of the container
+ * @filename  : the cgroup attribute filename
+ * @value     : the value to be set
+ * Returns 0 on success, < 0 otherwise
+ */
+extern int lxc_cgroup_set_bypath(const char *cgpath, const char *filename, const char *value);
 
 /*
  * Set a specified value for a specified subsystem. The specified
  * subsystem must be fully specified, eg. "cpu.shares"
  * @name      : the name of the container
- * @filename : the cgroup attribute filename
+ * @filename  : the cgroup attribute filename
  * @value     : the value to be set
+ * @lxcpath   : lxc config path for container
  * Returns 0 on success, < 0 otherwise
  */
-extern int lxc_cgroup_set(const char *name, const char *filename, const char *value);
+extern int lxc_cgroup_set(const char *name, const char *filename, const char *value, const char *lxcpath);
 
 /*
  * Get a specified value for a specified subsystem. The specified
  * subsystem must be fully specified, eg. "cpu.shares"
  * @name      : the name of the container
- * @filename : the cgroup attribute filename
+ * @filename  : the cgroup attribute filename
  * @value     : the value to be set
  * @len       : the len of the value variable
+ * @lxcpath   : lxc config path for container
  * Returns the number of bytes read, < 0 on error
  */
 extern int lxc_cgroup_get(const char *name, const char *filename,
-			  char *value, size_t len);
+			  char *value, size_t len, const char *lxcpath);
 
 /*
  * Retrieve the error string associated with the error returned by
@@ -157,14 +191,34 @@ extern int lxc_checkpoint(const char *name, int sfd, int flags);
  * @sfd: fd from which the container is restarted
  * @conf: lxc_conf structure.
  * @flags : restart flags (an ORed value)
+ * @lxcpath: container path
  * Returns 0 on success, < 0 otherwise
  */
-extern int lxc_restart(const char *, int, struct lxc_conf *, int);
+extern int lxc_restart(const char *, int, struct lxc_conf *, int, const char *);
 
 /*
- * Returns the version number of the library
+ * Create and return a new lxccontainer struct.
  */
-extern const char const *lxc_version(void);
+extern struct lxc_container *lxc_container_new(const char *name, const char *configpath);
+
+/*
+ * Returns 1 on success, 0 on failure.
+ */
+extern int lxc_container_get(struct lxc_container *c);
+
+/*
+ * Put a lxccontainer struct reference.
+ * Return -1 on error.
+ * Return 0 if this was not the last reference.
+ * If it is the last reference, free the lxccontainer and return 1.
+ */
+extern int lxc_container_put(struct lxc_container *c);
+
+/*
+ * Get a list of valid wait states.
+ * If states is NULL, simply return the number of states
+ */
+extern int lxc_get_wait_states(const char **states);
 
 #ifdef __cplusplus
 }
