@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2010
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -115,7 +115,7 @@ int main(int argc, char *argv[])
 	if (ret)
 		return ret;
 
-	ret = lxc_log_init(my_args.log_file, my_args.log_priority,
+	ret = lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
 			   my_args.progname, my_args.quiet);
 	if (ret)
 		return ret;
