@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,15 +31,17 @@
 #include <sys/stat.h>
 #include <sys/param.h>
 
-
+#include "caps.h"
 #include "lxc.h"
 #include "log.h"
 #include "conf.h"
 #include "confile.h"
 #include "arguments.h"
 #include "config.h"
+#include "start.h"
+#include "utils.h"
 
-lxc_log_define(lxc_execute_ui, lxc_start);
+lxc_log_define(lxc_execute_ui, lxc_execute);
 
 static struct lxc_list defines;
 
@@ -87,28 +89,29 @@ Options :\n\
 
 int main(int argc, char *argv[])
 {
-	static char **args;
 	char *rcfile;
 	struct lxc_conf *conf;
 
 	lxc_list_init(&defines);
 
+	if (lxc_caps_init())
+		return -1;
+
 	if (lxc_arguments_parse(&my_args, argc, argv))
 		return -1;
 
-	if (lxc_log_init(my_args.log_file, my_args.log_priority,
+	if (lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
 			 my_args.progname, my_args.quiet))
 		return -1;
 
-	args = lxc_arguments_dup(LXCLIBEXECDIR "/lxc-init", &my_args);
-	if (!args)
-		return -1;
-
 	/* rcfile is specified in the cli option */
 	if (my_args.rcfile)
 		rcfile = (char *)my_args.rcfile;
 	else {
-		if (!asprintf(&rcfile, LXCPATH "/%s/config", my_args.name)) {
+		int rc;
+
+		rc = asprintf(&rcfile, "%s/%s/config", my_args.lxcpath, my_args.name);
+		if (rc == -1) {
 			SYSERROR("failed to allocate memory");
 			return -1;
 		}
@@ -134,6 +137,5 @@ int main(int argc, char *argv[])
 	if (lxc_config_define_load(&defines, conf))
 		return -1;
 
-	return lxc_start(my_args.name, args, conf);
+	return lxc_execute(my_args.name, my_args.argv, my_args.quiet, conf, my_args.lxcpath);
 }
-
