@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2010
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,6 +36,7 @@
 #include "config.h"
 #include "confile.h"
 #include "arguments.h"
+#include "utils.h"
 
 lxc_log_define(lxc_restart_ui, lxc_restart);
 
@@ -122,7 +123,7 @@ int main(int argc, char *argv[])
 	if (lxc_arguments_parse(&my_args, argc, argv))
 		return -1;
 
-	if (lxc_log_init(my_args.log_file, my_args.log_priority,
+	if (lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
 			 my_args.progname, my_args.quiet))
 		return -1;
 
@@ -132,7 +133,7 @@ int main(int argc, char *argv[])
 	else {
 		int rc;
 
-		rc = asprintf(&rcfile, LXCPATH "/%s/config", my_args.name);
+		rc = asprintf(&rcfile, "%s/%s/config", my_args.lxcpath, my_args.name);
 		if (rc == -1) {
 			SYSERROR("failed to allocate memory");
 			return -1;
@@ -171,7 +172,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	ret = lxc_restart(my_args.name, sfd, conf, my_args.flags);
+	ret = lxc_restart(my_args.name, sfd, conf, my_args.flags, my_args.lxcpath);
 
 	if (my_args.statefile)
 		close(sfd);
