@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,14 +54,51 @@ lxc_log_define(lxc_start_ui, lxc_start);
 
 static struct lxc_list defines;
 
+static int ensure_path(char **confpath, const char *path)
+{
+	int err = -1, fd;
+	char *fullpath = NULL;
+
+	if (path) {
+		if (access(path, W_OK)) {
+			fd = creat(path, 0600);
+			if (fd < 0) {
+				SYSERROR("failed to create '%s'", path);
+				goto err;
+			}
+			close(fd);
+		}
+
+		fullpath = realpath(path, NULL);
+		if (!fullpath) {
+			SYSERROR("failed to get the real path of '%s'", path);
+			goto err;
+		}
+
+		*confpath = strdup(fullpath);
+		if (!*confpath) {
+			ERROR("failed to dup string '%s'", fullpath);
+			goto err;
+		}
+	}
+	err = 0;
+
+err:
+	if (fullpath)
+		free(fullpath);
+	return err;
+}
+
 static int my_parser(struct lxc_arguments* args, int c, char* arg)
 {
 	switch (c) {
 	case 'c': args->console = arg; break;
+	case 'L': args->console_log = arg; break;
 	case 'd': args->daemonize = 1; args->close_all_fds = 1; break;
 	case 'f': args->rcfile = arg; break;
 	case 'C': args->close_all_fds = 1; break;
 	case 's': return lxc_config_define_add(&defines, arg);
+	case 'p': args->pidfile = arg; break;
 	}
 	return 0;
 }
@@ -71,7 +108,9 @@ static const struct option my_longopts[] = {
 	{"rcfile", required_argument, 0, 'f'},
 	{"define", required_argument, 0, 's'},
 	{"console", required_argument, 0, 'c'},
+	{"console-log", required_argument, 0, 'L'},
 	{"close-all-fds", no_argument, 0, 'C'},
+	{"pidfile", required_argument, 0, 'p'},
 	LXC_COMMON_OPTIONS
 };
 
@@ -83,18 +122,21 @@ static struct lxc_arguments my_args = {
 lxc-start start COMMAND in specified container NAME\n\
 \n\
 Options :\n\
-  -n, --name=NAME      NAME for name of the container\n\
-  -d, --daemon         daemonize the container\n\
-  -f, --rcfile=FILE    Load configuration file FILE\n\
-  -c, --console=FILE   Set the file output for the container console\n\
-  -C, --close-all-fds  If any fds are inherited, close them\n\
-                       If not specified, exit with failure instead\n\
-		       Note: --daemon implies --close-all-fds\n\
-  -s, --define KEY=VAL Assign VAL to configuration variable KEY\n",
+  -n, --name=NAME        NAME for name of the container\n\
+  -d, --daemon           daemonize the container\n\
+  -p, --pidfile=FILE     Create a file with the process id\n\
+  -f, --rcfile=FILE      Load configuration file FILE\n\
+  -c, --console=FILE     Use specified FILE for the container console\n\
+  -L, --console-log=FILE Log container console output to FILE\n\
+  -C, --close-all-fds    If any fds are inherited, close them\n\
+                         If not specified, exit with failure instead\n\
+		         Note: --daemon implies --close-all-fds\n\
+  -s, --define KEY=VAL   Assign VAL to configuration variable KEY\n",
 	.options   = my_longopts,
 	.parser    = my_parser,
 	.checker   = NULL,
 	.daemonize = 0,
+	.pidfile = NULL,
 };
 
 int main(int argc, char *argv[])
@@ -107,6 +149,7 @@ int main(int argc, char *argv[])
 		"/sbin/init",
 		'\0',
 	};
+	FILE *pid_fp = NULL;
 
 	lxc_list_init(&defines);
 
@@ -117,30 +160,26 @@ int main(int argc, char *argv[])
 		return err;
 
 	if (!my_args.argc)
-		args = default_args; 
+		args = default_args;
 	else
 		args = my_args.argv;
 
-	if (lxc_log_init(my_args.log_file, my_args.log_priority,
+	if (lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
 			 my_args.progname, my_args.quiet))
 		return err;
 
-	if (putenv("container=lxc")) {
-		SYSERROR("failed to set environment variable");
-		return err;
-	}
-
 	/* rcfile is specified in the cli option */
 	if (my_args.rcfile)
 		rcfile = (char *)my_args.rcfile;
 	else {
 		int rc;
 
-		rc = asprintf(&rcfile, LXCPATH "/%s/config", my_args.name);
+		rc = asprintf(&rcfile, "%s/%s/config", my_args.lxcpath, my_args.name);
 		if (rc == -1) {
 			SYSERROR("failed to allocate memory");
 			return err;
 		}
+		INFO("using rcfile %s", rcfile);
 
 		/* container configuration does not exist */
 		if (access(rcfile, F_OK)) {
@@ -168,35 +207,23 @@ int main(int argc, char *argv[])
 		return err;
 	}
 
-	if (my_args.console) {
+	if (ensure_path(&conf->console.path, my_args.console) < 0) {
+		ERROR("failed to ensure console path '%s'", my_args.console);
+		return err;
+	}
 
-		char *console, fd;
+	if (ensure_path(&conf->console.log_path, my_args.console_log) < 0) {
+		ERROR("failed to ensure console log '%s'", my_args.console_log);
+		return err;
+	}
 
-		if (access(my_args.console, W_OK)) {
-
-			fd = creat(my_args.console, 0600);
-			if (fd < 0) {
-				SYSERROR("failed to touch file '%s'",
-					 my_args.console);
-				return err;
-			}
-			close(fd);
-		}
-
-		console = realpath(my_args.console, NULL);
-		if (!console) {
-			SYSERROR("failed to get the real path of '%s'",
-				 my_args.console);
+	if (my_args.pidfile != NULL) {
+		pid_fp = fopen(my_args.pidfile, "w");
+		if (pid_fp == NULL) {
+			SYSERROR("failed to create pidfile '%s' for '%s'",
+				 my_args.pidfile, my_args.name);
 			return err;
 		}
-
-		conf->console.path = strdup(console);
-		if (!conf->console.path) {
-			ERROR("failed to dup string '%s'", console);
-			return err;
-		}
-
-		free(console);
 	}
 
 	if (my_args.daemonize) {
@@ -214,10 +241,18 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (pid_fp != NULL) {
+		if (fprintf(pid_fp, "%d\n", getpid()) < 0) {
+			SYSERROR("failed to write '%s'", my_args.pidfile);
+			return err;
+		}
+		fclose(pid_fp);
+	}
+
 	if (my_args.close_all_fds)
 		conf->close_all_fds = 1;
 
-	err = lxc_start(my_args.name, args, conf);
+	err = lxc_start(my_args.name, args, conf, my_args.lxcpath);
 
 	/*
 	 * exec ourself, that requires to have all opened fd
@@ -230,6 +265,9 @@ int main(int argc, char *argv[])
 		err = -1;
 	}
 
+	if (my_args.pidfile)
+		unlink(my_args.pidfile);
+
 	return err;
 }
 
