@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,12 +54,9 @@ void usage(char *cmd)
 
 static uid_t lookup_user(const char *optarg)
 {
-	int bufflen = sysconf(_SC_GETPW_R_SIZE_MAX);
-	char buff[bufflen];
 	char name[sysconf(_SC_LOGIN_NAME_MAX)];
 	uid_t uid = -1;
-	struct passwd pwent;
-	struct passwd *pent;
+	struct passwd *pwent = NULL;
 
 	if (!optarg || (optarg[0] == '\0'))
 		return uid;
@@ -69,13 +66,15 @@ static uid_t lookup_user(const char *optarg)
 		if (sscanf(optarg, "%s", name) < 1)
 			return uid;
 
-		if (getpwnam_r(name, &pwent, buff, bufflen, &pent) || !pent) {
+		pwent = getpwnam(name);
+		if (!pwent) {
 			ERROR("invalid username %s", name);
 			return uid;
 		}
-		uid = pent->pw_uid;
+		uid = pwent->pw_uid;
 	} else {
-		if (getpwuid_r(uid, &pwent, buff, bufflen, &pent) || !pent) {
+		pwent = getpwuid(uid);
+		if (!pwent) {
 			ERROR("invalid uid %d", uid);
 			uid = -1;
 			return uid;
@@ -84,51 +83,6 @@ static uid_t lookup_user(const char *optarg)
 	return uid;
 }
 
-static char *namespaces_list[] = {
-	"MOUNT", "PID", "UTSNAME", "IPC",
-	"USER", "NETWORK"
-};
-static int cloneflags_list[] = {
-	CLONE_NEWNS, CLONE_NEWPID, CLONE_NEWUTS, CLONE_NEWIPC,
-	CLONE_NEWUSER, CLONE_NEWNET
-};
-
-static int lxc_namespace_2_cloneflag(char *namespace)
-{
-	int i, len;
-	len = sizeof(namespaces_list)/sizeof(namespaces_list[0]);
-	for (i = 0; i < len; i++)
-		if (!strcmp(namespaces_list[i], namespace))
-			return cloneflags_list[i];
-
-	ERROR("invalid namespace name %s", namespace);
-	return -1;
-}
-
-static int lxc_fill_namespace_flags(char *flaglist, int *flags)
-{
-	char *token, *saveptr = NULL;
-	int aflag;
-
-	if (!flaglist) {
-		ERROR("need at least one namespace to unshare");
-		return -1;
-	}
-
-	token = strtok_r(flaglist, "|", &saveptr);
-	while (token) {
-
-		aflag = lxc_namespace_2_cloneflag(token);
-		if (aflag < 0)
-			return -1;
-
-		*flags |= aflag;
-
-		token = strtok_r(NULL, "|", &saveptr);
-	}
-	return 0;
-}
-
 
 struct start_arg {
 	char ***args;
@@ -158,7 +112,6 @@ int main(int argc, char *argv[])
 {
 	int opt, status;
 	int ret;
-	char *pid_name;
 	char *namespaces = NULL;
 	char **args;
 	int flags = 0;
@@ -171,11 +124,13 @@ int main(int argc, char *argv[])
 		.flags = &flags,
 	};
 
-	while ((opt = getopt(argc, argv, "s:u:")) != -1) {
+	while ((opt = getopt(argc, argv, "s:u:h")) != -1) {
 		switch (opt) {
 		case 's':
 			namespaces = optarg;
 			break;
+		case 'h':
+			usage(argv[0]);
 		case 'u':
 			uid = lookup_user(optarg);
 			if (uid == -1)
@@ -183,10 +138,10 @@ int main(int argc, char *argv[])
 		}
 	}
 
-    if (argv[optind] == NULL) {
-        ERROR("a command to execute in the new namespace is required");
-        return 1;
-    }
+	if (argv[optind] == NULL) {
+		ERROR("a command to execute in the new namespace is required");
+		return 1;
+	}
 
 	args = &argv[optind];
 
@@ -194,8 +149,8 @@ int main(int argc, char *argv[])
 	if (ret)
 		return ret;
 
-        ret = lxc_fill_namespace_flags(namespaces, &flags);
- 	if (ret)
+	ret = lxc_fill_namespace_flags(namespaces, &flags);
+	if (ret)
 		usage(argv[0]);
 
 	if (!(flags & CLONE_NEWUSER) && uid != -1) {
@@ -214,14 +169,5 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
-	if (lxc_ns_is_mounted()) {
-		if (asprintf(&pid_name, "%d", pid) == -1) {
-			ERROR("pid_name: failed to allocate memory");
-			return -1;
-		}
-		lxc_cgroup_destroy(pid_name);
-		free(pid_name);
-	}
-
 	return  lxc_error_set_and_log(pid, status);
 }
