@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -47,13 +47,23 @@ lxc_log_define(lxc_monitor, lxc);
 #define UNIX_PATH_MAX 108
 #endif
 
-static void lxc_monitor_send(struct lxc_msg *msg)
+static void lxc_monitor_send(struct lxc_msg *msg, const char *lxcpath)
 {
 	int fd;
 	struct sockaddr_un addr = { .sun_family = AF_UNIX };
 	char *offset = &addr.sun_path[1];
+	size_t ret, len;
 
-	strcpy(offset, "lxc-monitor");
+	/*
+	 * addr.sun_path is only 108 bytes.
+	 * should we take a hash of lxcpath?  a subset of it?
+	 */
+	len = sizeof(addr.sun_path) - 1;
+	ret = snprintf(offset, len, "%s/lxc-monitor", lxcpath);
+	if (ret < 0 || ret >= len) {
+		ERROR("lxcpath too long to open monitor");
+		return;
+	}
 
 	fd = socket(PF_UNIX, SOCK_DGRAM, 0);
 	if (fd < 0)
@@ -65,22 +75,33 @@ static void lxc_monitor_send(struct lxc_msg *msg)
 	close(fd);
 }
 
-void lxc_monitor_send_state(const char *name, lxc_state_t state)
+void lxc_monitor_send_state(const char *name, lxc_state_t state, const char *lxcpath)
 {
 	struct lxc_msg msg = { .type = lxc_msg_state,
 			       .value = state };
 	strncpy(msg.name, name, sizeof(msg.name));
+	msg.name[sizeof(msg.name) - 1] = 0;
 
-	lxc_monitor_send(&msg);
+	lxc_monitor_send(&msg, lxcpath);
 }
 
-int lxc_monitor_open(void)
+int lxc_monitor_open(const char *lxcpath)
 {
 	struct sockaddr_un addr = { .sun_family = AF_UNIX };
 	char *offset = &addr.sun_path[1];
 	int fd;
+	size_t ret, len;
 
-	strcpy(offset, "lxc-monitor");
+	/*
+	 * addr.sun_path is only 108 bytes.
+	 * should we take a hash of lxcpath?  a subset of it?
+	 */
+	len = sizeof(addr.sun_path) - 1;
+	ret = snprintf(offset, len, "%s/lxc-monitor", lxcpath);
+	if (ret < 0 || ret >= len) {
+		ERROR("lxcpath too long to open monitor");
+		return -1;
+	}
 
 	fd = socket(PF_UNIX, SOCK_DGRAM, 0);
 	if (fd < 0) {
@@ -97,11 +118,28 @@ int lxc_monitor_open(void)
 	return fd;
 }
 
-int lxc_monitor_read(int fd, struct lxc_msg *msg)
+/* timeout of 0 means return immediately;  -1 means wait forever */
+int lxc_monitor_read_timeout(int fd, struct lxc_msg *msg, int timeout)
 {
 	struct sockaddr_un from;
 	socklen_t len = sizeof(from);
 	int ret;
+	fd_set rfds;
+	struct timeval tv;
+
+	if (timeout != -1) {
+		FD_ZERO(&rfds);
+		FD_SET(fd, &rfds);
+
+		tv.tv_sec = timeout;
+		tv.tv_usec = 0;
+
+		ret = select(fd+1, &rfds, NULL, NULL, &tv);
+		if (ret == -1)
+			return -1;
+		else if (!ret)
+			return -2;  // timed out
+	}
 
 	ret = recvfrom(fd, msg, sizeof(*msg), 0,
 		       (struct sockaddr *)&from, &len);
@@ -113,6 +151,11 @@ int lxc_monitor_read(int fd, struct lxc_msg *msg)
 	return ret;
 }
 
+int lxc_monitor_read(int fd, struct lxc_msg *msg)
+{
+	return lxc_monitor_read_timeout(fd, msg, -1);
+}
+
 int lxc_monitor_close(int fd)
 {
 	return close(fd);
