@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2009
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,6 @@
 #include <alloca.h>
 #include <errno.h>
 #include <signal.h>
-#include <syscall.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -34,8 +33,6 @@
 #include "namespace.h"
 #include "log.h"
 
-#include "setns.h"
-
 lxc_log_define(lxc_namespace, lxc);
 
 struct clone_arg {
@@ -43,16 +40,6 @@ struct clone_arg {
 	void *arg;
 };
 
-int setns(int nstype, int fd)
-{
-#ifndef __NR_setns
-	errno = ENOSYS;
-	return -1;
-#else
-	return syscall(__NR_setns, nstype, fd);
-#endif
-}
-
 static int do_clone(void *arg)
 {
 	struct clone_arg *clone_arg = arg;
@@ -67,14 +54,14 @@ pid_t lxc_clone(int (*fn)(void *), void *arg, int flags)
 	};
 
 	long stack_size = sysconf(_SC_PAGESIZE);
- 	void *stack = alloca(stack_size) + stack_size;
+	void *stack = alloca(stack_size);
 	pid_t ret;
 
 #ifdef __ia64__
 	ret = __clone2(do_clone, stack,
 		       stack_size, flags | SIGCHLD, &clone_arg);
 #else
-	ret = clone(do_clone, stack, flags | SIGCHLD, &clone_arg);
+	ret = clone(do_clone, stack  + stack_size, flags | SIGCHLD, &clone_arg);
 #endif
 	if (ret < 0)
 		ERROR("failed to clone(0x%x): %s", flags, strerror(errno));
@@ -82,37 +69,47 @@ pid_t lxc_clone(int (*fn)(void *), void *arg, int flags)
 	return ret;
 }
 
-int lxc_attach(pid_t pid)
-{
-	char path[MAXPATHLEN];
-	char *ns[] = { "pid", "mnt", "net", "ipc", "uts" };
-	const int size = sizeof(ns) / sizeof(char *);
-	int fd[size];
-	int i;
+static char *namespaces_list[] = {
+	"MOUNT", "PID", "UTSNAME", "IPC",
+	"USER", "NETWORK"
+};
+static int cloneflags_list[] = {
+	CLONE_NEWNS, CLONE_NEWPID, CLONE_NEWUTS, CLONE_NEWIPC,
+	CLONE_NEWUSER, CLONE_NEWNET
+};
 
-	sprintf(path, "/proc/%d/ns", pid);
-	if (access(path, R_OK)) {
-		ERROR("Does this kernel version support 'attach' ?");
+int lxc_namespace_2_cloneflag(char *namespace)
+{
+	int i, len;
+	len = sizeof(namespaces_list)/sizeof(namespaces_list[0]);
+	for (i = 0; i < len; i++)
+		if (!strcmp(namespaces_list[i], namespace))
+			return cloneflags_list[i];
+
+	ERROR("invalid namespace name %s", namespace);
+	return -1;
+}
+
+int lxc_fill_namespace_flags(char *flaglist, int *flags)
+{
+	char *token, *saveptr = NULL;
+	int aflag;
+
+	if (!flaglist) {
+		ERROR("need at least one namespace to unshare");
 		return -1;
 	}
 
-	for (i = 0; i < size; i++) {
-		sprintf(path, "/proc/%d/ns/%s", pid, ns[i]);
-		fd[i] = open(path, O_RDONLY);
-		if (fd[i] < 0) {
-			SYSERROR("failed to open '%s'", path);
+	token = strtok_r(flaglist, "|", &saveptr);
+	while (token) {
+
+		aflag = lxc_namespace_2_cloneflag(token);
+		if (aflag < 0)
 			return -1;
-		}
+
+		*flags |= aflag;
+
+		token = strtok_r(NULL, "|", &saveptr);
 	}
-
-	for (i = 0; i < size; i++) {
-		if (setns(0, fd[i])) {
-			SYSERROR("failed to set namespace '%s'", ns[i]);
-			return -1;
-		}
-
-		close(fd[i]);
-	}
-
 	return 0;
 }
