@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2009
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,9 +23,11 @@
 #ifndef __namespace_h
 #define __namespace_h
 
-#include <syscall.h>
+#include <sys/syscall.h>
 #include <sched.h>
 
+#include "config.h"
+
 #ifndef CLONE_FS
 #  define CLONE_FS                0x00000200
 #endif
@@ -47,11 +49,23 @@
 #ifndef CLONE_NEWNET
 #  define CLONE_NEWNET            0x40000000
 #endif
+
+#if defined(__ia64__)
+int __clone2(int (*__fn) (void *__arg), void *__child_stack_base,
+             size_t __child_stack_size, int __flags, void *__arg, ...);
+#elif defined(IS_BIONIC)
+int clone(int (*fn)(void *), void *child_stack,
+	int flags, void *arg);
+#else
 int clone(int (*fn)(void *), void *child_stack,
 	int flags, void *arg, ...
 	/* pid_t *ptid, struct user_desc *tls, pid_t *ctid */ );
+#endif
 
 
 extern pid_t lxc_clone(int (*fn)(void *), void *arg, int flags);
 
+extern int lxc_namespace_2_cloneflag(char *namespace);
+extern int lxc_fill_namespace_flags(char *flaglist, int *flags);
+
 #endif
