@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,8 @@
 #include <stdlib.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
-#include <nl.h>
+
+#include "nl.h"
 
 #define NLMSG_TAIL(nmsg) \
         ((struct rtattr *) (((void *) (nmsg)) + NLMSG_ALIGN((nmsg)->nlmsg_len)))
@@ -47,7 +48,7 @@ extern void *nlmsg_data(struct nlmsg *nlmsg)
 	return data;
 }
 
-static int nla_put(struct nlmsg *nlmsg, int attr, 
+static int nla_put(struct nlmsg *nlmsg, int attr,
 		   const void *data, size_t len)
 {
 	struct rtattr *rta;
@@ -62,7 +63,7 @@ static int nla_put(struct nlmsg *nlmsg, int attr,
 	return 0;
 }
 
-extern int nla_put_buffer(struct nlmsg *nlmsg, int attr, 
+extern int nla_put_buffer(struct nlmsg *nlmsg, int attr,
 			  const void *data, size_t size)
 {
 	return nla_put(nlmsg, attr, data, size);
@@ -78,7 +79,7 @@ extern int nla_put_u32(struct nlmsg *nlmsg, int attr, int value)
 	return nla_put(nlmsg, attr, &value, sizeof(value));
 }
 
-extern int nla_put_u16(struct nlmsg *nlmsg, int attr, ushort value)
+extern int nla_put_u16(struct nlmsg *nlmsg, int attr, unsigned short value)
 {
 	return nla_put(nlmsg, attr, &value, 2);
 }
@@ -155,7 +156,8 @@ again:
 	if (!ret)
 		return 0;
 
-	if (msg.msg_flags & MSG_TRUNC)
+	if (msg.msg_flags & MSG_TRUNC &&
+	    ret == answer->nlmsghdr.nlmsg_len)
 		return -EMSGSIZE;
 
 	return ret;
@@ -182,9 +184,8 @@ extern int netlink_send(struct nl_handler *handler, struct nlmsg *nlmsg)
         nladdr.nl_groups = 0;
 
 	ret = sendmsg(handler->fd, &msg, 0);
-	if (ret < 0) {
+	if (ret < 0)
 		return -errno;
-	}
 
 	return ret;
 }
@@ -192,10 +193,9 @@ extern int netlink_send(struct nl_handler *handler, struct nlmsg *nlmsg)
 #ifndef NLMSG_ERROR
 #define NLMSG_ERROR                0x2
 #endif
-extern int netlink_transaction(struct nl_handler *handler, 
+extern int netlink_transaction(struct nl_handler *handler,
 			       struct nlmsg *request, struct nlmsg *answer)
 {
-
 	int ret;
 
 	ret = netlink_send(handler, request);
@@ -208,12 +208,9 @@ extern int netlink_transaction(struct nl_handler *handler,
 
 	if (answer->nlmsghdr.nlmsg_type == NLMSG_ERROR) {
 		struct nlmsgerr *err = (struct nlmsgerr*)NLMSG_DATA(answer);
-		errno = -err->error;
-		if (errno)
-			perror("Error configuring kernel");
-		return -errno;
+		return err->error;
 	}
-	
+
 	return 0;
 }
 
@@ -229,11 +226,11 @@ extern int netlink_open(struct nl_handler *handler, int protocol)
         if (handler->fd < 0)
                 return -errno;
 
-        if (setsockopt(handler->fd, SOL_SOCKET, SO_SNDBUF, 
+        if (setsockopt(handler->fd, SOL_SOCKET, SO_SNDBUF,
 		       &sndbuf, sizeof(sndbuf)) < 0)
                 return -errno;
 
-        if (setsockopt(handler->fd, SOL_SOCKET, SO_RCVBUF, 
+        if (setsockopt(handler->fd, SOL_SOCKET, SO_RCVBUF,
 		       &rcvbuf,sizeof(rcvbuf)) < 0)
                 return -errno;
 
@@ -241,12 +238,12 @@ extern int netlink_open(struct nl_handler *handler, int protocol)
         handler->local.nl_family = AF_NETLINK;
         handler->local.nl_groups = 0;
 
-        if (bind(handler->fd, (struct sockaddr*)&handler->local, 
+        if (bind(handler->fd, (struct sockaddr*)&handler->local,
 		 sizeof(handler->local)) < 0)
                 return -errno;
 
         socklen = sizeof(handler->local);
-        if (getsockname(handler->fd, (struct sockaddr*)&handler->local, 
+        if (getsockname(handler->fd, (struct sockaddr*)&handler->local,
 			&socklen) < 0)
                 return -errno;
 
