@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,8 +29,27 @@
 #include <dirent.h>
 
 #include "parse.h"
+#include "config.h"
 #include <lxc/log.h>
 
+/* Define getline() if missing from the C library */
+#ifndef HAVE_GETLINE
+#ifdef HAVE_FGETLN
+#include <../include/getline.h>
+#endif
+#endif
+
+/* Workaround for the broken signature of alphasort() in bionic.
+   This was fixed upstream in 40e467ec668b59be25491bd44bf348a884d6a68d so the
+   workaround can probably be dropped with the next version of the Android NDK.
+ */
+#ifdef IS_BIONIC
+int bionic_alphasort(const struct dirent** a, const struct dirent** b) {
+       return strcoll((*a)->d_name, (*b)->d_name);
+}
+#endif
+
+
 lxc_log_define(lxc_parse, lxc);
 
 static int dir_filter(const struct dirent *dirent)
@@ -47,7 +66,11 @@ int lxc_dir_for_each(const char *name, const char *directory,
 	struct dirent **namelist;
 	int n, ret = 0;
 
+#ifdef IS_BIONIC
+	n = scandir(directory, &namelist, dir_filter, bionic_alphasort);
+#else
 	n = scandir(directory, &namelist, dir_filter, alphasort);
+#endif
 	if (n < 0) {
 		SYSERROR("failed to scan %s directory", directory);
 		return -1;
@@ -81,10 +104,8 @@ int lxc_file_for_each_line(const char *file, lxc_file_cb callback, void *data)
 
 	while (getline(&line, &len, f) != -1) {
 		err = callback(line, data);
-		if (err) {
-			ERROR("failed to process '%s'", line);
+		if (err)
 			break;
-		}
 	}
 
 	if (line)
