@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,13 +26,13 @@
 typedef int (*lxc_dir_cb)(const char *name, const char *directory,
 			  const char *file, void *data);
 
-typedef int (*lxc_file_cb)(void *buffer, void *data);
+typedef int (*lxc_file_cb)(char *buffer, void *data);
 
 extern int lxc_dir_for_each(const char *name, const char *directory,
 			    lxc_dir_cb callback, void *data);
 
 extern int lxc_file_for_each_line(const char *file, lxc_file_cb callback,
-				  char *buffer, size_t len, void* data);
+				  void* data);
 
 extern int lxc_char_left_gc(char *buffer, size_t len);
 
