@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -131,13 +131,15 @@ extern int lxc_setup_fs(void)
 	if (mount_fs("proc", "/proc", "proc"))
 		return -1;
 
+	/* if we can't mount /dev/shm, continue anyway */
 	if (mount_fs("shmfs", "/dev/shm", "tmpfs"))
-		return -1;
+		INFO("failed to mount /dev/shm");
 
 	/* If we were able to mount /dev/shm, then /dev exists */
+	/* Sure, but it's read-only per config :) */
 	if (access("/dev/mqueue", F_OK) && mkdir("/dev/mqueue", 0666)) {
-		SYSERROR("failed to create '/dev/mqueue'");
-		return -1;
+		DEBUG("failed to create '/dev/mqueue'");
+		return 0;
 	}
 
 	if (mount_fs("mqueue", "/dev/mqueue", "mqueue"))
@@ -147,7 +149,7 @@ extern int lxc_setup_fs(void)
 }
 
 /* borrowed from iproute2 */
-extern int get_u16(ushort *val, const char *arg, int base)
+extern int get_u16(unsigned short *val, const char *arg, int base)
 {
 	unsigned long res;
 	char *ptr;
@@ -191,3 +193,60 @@ extern int mkdir_p(char *dir, mode_t mode)
 
         return 0;
 }
+
+static char *copypath(char *p)
+{
+	int len = strlen(p);
+	char *retbuf;
+
+	if (len < 1)
+		return NULL;
+	if (p[len-1] == '\n') {
+		p[len-1] = '\0';
+		len--;
+	}
+	retbuf = malloc(len+1);
+	if (!retbuf)
+		return NULL;
+	strcpy(retbuf, p);
+	return retbuf;
+}
+
+char *default_lxcpath;
+
+const char *default_lxc_path(void)
+{
+	char buf[1024], *p;
+	FILE *fin;
+
+	if (default_lxcpath)
+		return default_lxcpath;
+
+	fin = fopen(LXC_GLOBAL_CONF, "r");
+	if (fin) {
+		while (fgets(buf, 1024, fin)) {
+			if (buf[0] == '#')
+				continue;
+			p = strstr(buf, "lxcpath");
+			if (!p)
+				continue;
+			p = strchr(p, '=');
+			if (!p)
+				continue;
+			p++;
+			while (*p && (*p == ' ' || *p == '\t')) p++;
+			if (!*p)
+				continue;
+			default_lxcpath = copypath(p);
+			goto out;
+		}
+	}
+	/* we couldn't open the file, or didn't find a lxcpath
+	 * entry there.  Return @LXCPATH@ */
+	default_lxcpath = LXCPATH;
+
+out:
+	if (fin)
+		fclose(fin);
+	return default_lxcpath;
+}
