@@ -4,7 +4,7 @@
  * (C) Copyright IBM Corp. 2007, 2008
  *
  * Authors:
- * Daniel Lezcano <dlezcano at fr.ibm.com>
+ * Daniel Lezcano <daniel.lezcano at free.fr>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,34 +23,14 @@
 #ifndef _utils_h
 #define _utils_h
 
-#define LXC_TTY_HANDLER(s) \
-	static struct sigaction lxc_tty_sa_##s;				\
-	static void tty_##s##_handler(int sig, siginfo_t *info, void *ctx) \
-	{								\
-		if (lxc_tty_sa_##s.sa_handler == SIG_DFL ||		\
-		    lxc_tty_sa_##s.sa_handler == SIG_IGN)		\
-			return;						\
-		(*lxc_tty_sa_##s.sa_sigaction)(sig, info, ctx);	\
-	}
-
-#define LXC_TTY_ADD_HANDLER(s) \
-	do { \
-		struct sigaction sa; \
-		sa.sa_sigaction = tty_##s##_handler; \
-		sa.sa_flags = SA_SIGINFO; \
-		sigfillset(&sa.sa_mask); \
-		/* No error expected with sigaction. */ \
-		sigaction(s, &sa, &lxc_tty_sa_##s); \
-	} while (0)
-
-#define LXC_TTY_DEL_HANDLER(s) \
-	do { \
-		sigaction(s, &lxc_tty_sa_##s, NULL); \
-	} while (0)
-
-#endif
-
 extern int lxc_copy_file(const char *src, const char *dst);
 extern int lxc_setup_fs(void);
-extern int get_u16(ushort *val, const char *arg, int base);
+extern int get_u16(unsigned short *val, const char *arg, int base);
 extern int mkdir_p(const char *dir, mode_t mode);
+/*
+ * Return a newly allocated buffer containing the default container
+ * path.  Caller must free this buffer.
+ */
+extern const char *default_lxc_path(void);
+
+#endif
